@@ -5,16 +5,14 @@ import os
 import re
 from typing import Self, Callable, Sequence, Hashable, Any
 
-from utils import default_post_add_hook
-
 logger = logging.getLogger(__name__)
 
 
-class BKTreeNode[T]:
-    def __init__(self, value: T, value_type: type[T] = str):
-        self.value: T = value
+class BKTreeNode[ValueT]:
+    def __init__(self, value: ValueT, value_type: type[ValueT] = str):
+        self.value: ValueT = value
         self.children: dict[int, Self] = {}
-        self.value_type: type[T] = value_type
+        self.value_type: type[ValueT] = value_type
 
     def __eq__(self, other):
         """Check the equality between two BKTree nodes.
@@ -22,7 +20,7 @@ class BKTreeNode[T]:
         Two nodes are equals when their value is the same and their children are equals (at the same distance).
 
         Args:
-            other (BKTreeNode[T]): The second node.
+            other (BKTreeNode[ValueT]): The second node.
 
         Returns:
             True if the two nodes are equals.
@@ -43,7 +41,7 @@ class BKTreeNode[T]:
         return True
 
     @classmethod
-    def from_dict(cls, node_input, value_type: type[T] = str):
+    def from_dict(cls, node_input, value_type: type[ValueT] = str):
         if value_type is str:
             new_node = cls(value=node_input["value"], value_type=value_type)
         else:
@@ -73,21 +71,23 @@ class BKTreeNode[T]:
         }
 
 
-class BKTree[T]:
+class BKTree[ValueT, NodeT]:
 
-    def __init__(self,
-                 distance_function: Callable[[Sequence[Hashable], Sequence[Hashable]], int],
-                 max_distance: int = 0,
-                 key: Callable[[T], str] | None = None,
-                 value_type: type[T] = str,
-                 node_type: type[BKTreeNode] = BKTreeNode,
-                 *,
-                 post_add_hook: Callable[[Self, BKTreeNode], None] = lambda tree, node: None,
-                 post_already_added_hook: Callable[[Self, BKTreeNode], None] = lambda tree, node: None,
-                 post_filter_add_hook: Callable[[Self, BKTreeNode], None] = lambda tree, node: None,
-                 post_insert_subtree_hook: Callable[[Self, BKTreeNode], None] = lambda tree, node: None,
-                 post_combine_hook: Callable[[Self, BKTreeNode], None] = lambda tree, node: None):
-        self.root: BKTreeNode[T] | None = None
+    def __init__(
+            self,
+            distance_function: Callable[[Sequence[Hashable], Sequence[Hashable]], int],
+            max_distance: int = 0,
+            key: Callable[[ValueT], str] | None = None,
+            value_type: type[ValueT] = str,
+            node_type: type[NodeT] = BKTreeNode,
+            *,
+            post_add_hook: Callable[[Self, NodeT], None] = lambda tree, node: None,
+            post_already_added_hook: Callable[[Self, NodeT], None] = lambda tree, node: None,
+            post_filter_add_hook: Callable[[Self, NodeT], None] = lambda tree, node: None,
+            post_insert_subtree_hook: Callable[[Self, NodeT], None] = lambda tree, node: None,
+            post_combine_hook: Callable[[Self, NodeT], None] = lambda tree, node: None
+    ):
+        self.root: BKTreeNode[ValueT] | None = None
         self.distance_function = distance_function
         self.max_distance = max_distance
         self.nb_elements = 0
@@ -102,13 +102,13 @@ class BKTree[T]:
 
     @classmethod
     def from_json(cls, filepath: os.PathLike | str, distance_function: Callable[[Sequence[Hashable], Sequence[Hashable]], int],
-                  key: Callable[[T], str] | None = None) -> Self:
+                  key: Callable[[ValueT], str] | None = None) -> Self:
         """Create a BK-Tree from a JSON file.
 
         Args:
             filepath (os.PathLike): The path to the JSON file.
             distance_function (Callable): A function that calculates the distance between two song identifiers
-            key (Callable[[T], str] | None): A callable that return the string to be compared.
+            key (Callable[[ValueT], str] | None): A callable that return the string to be compared.
 
         Returns:
             CustomBKTree: The BK-Tree.
@@ -120,7 +120,8 @@ class BKTree[T]:
 
     @classmethod
     def create_tree(cls, json_data: Any, distance_function: Callable[[Sequence[Hashable], Sequence[Hashable]], int],
-                    key: Callable[[T], str] | None = None, value_type: type[T] = str, node_type: type[BKTreeNode] = BKTreeNode) -> Self:
+                    key: Callable[[ValueT], str] | None = None, value_type: type[ValueT] = str,
+                    node_type: type[BKTreeNode] = BKTreeNode) -> Self:
         tree = cls(distance_function, json_data["max_distance"], key=key, value_type=value_type, node_type=node_type)
         tree.nb_elements = json_data["nb_elements"]
         tree.root = node_type.from_dict(json_data["nodes"], value_type=value_type)
@@ -143,7 +144,7 @@ class BKTree[T]:
         else:
             return 0
 
-    def __contains__(self, item: T):
+    def __contains__(self, item: ValueT):
         if self.root is None:
             return False
 
@@ -229,7 +230,7 @@ class BKTree[T]:
 
         return results
 
-    def add(self, value: T) -> tuple[BKTreeNode | None, bool]:
+    def add(self, value: ValueT) -> tuple[BKTreeNode | None, bool]:
         """Add a value to the BK-Tree. If the tree is empty, the value becomes the root.
 
         Args:
@@ -264,7 +265,7 @@ class BKTree[T]:
                     self.post_already_added_hook(self, search_result[0])
                     return search_result[0], False
 
-    def _add(self, node: BKTreeNode, value: T) -> BKTreeNode:
+    def _add(self, node: BKTreeNode, value: ValueT) -> BKTreeNode:
         """Recursively adds the song to the correct position in the BK-Tree.
 
         Args:
@@ -311,7 +312,7 @@ class BKTree[T]:
 
         return filtered_tree
 
-    def insert(self, value: T, start_node: BKTreeNode) -> BKTreeNode | None:
+    def insert(self, value: ValueT, start_node: BKTreeNode) -> BKTreeNode | None:
         """Insert a value in a subtree.
 
         Args:
@@ -353,7 +354,7 @@ class BKTree[T]:
                 logger.debug(f"{self.key(search_result[0].value)} is similar to {self.key(value)} and is already in the tree.")
                 return search_result[0]
 
-    def search_node_and_parent(self, value: T) -> tuple[BKTreeNode | None, BKTreeNode | None, int | None]:
+    def search_node_and_parent(self, value: ValueT) -> tuple[BKTreeNode | None, BKTreeNode | None, int | None]:
         """Get the node containing the value and his parent node.
 
         Args:
@@ -367,7 +368,7 @@ class BKTree[T]:
         """
         return self._search_node_and_parent(value, self.root)
 
-    def _search_node_and_parent(self, value: T, node: BKTreeNode, parent: BKTreeNode | None = None,
+    def _search_node_and_parent(self, value: ValueT, node: BKTreeNode, parent: BKTreeNode | None = None,
                                 distance_parent: int | None = None) -> tuple[BKTreeNode | None, BKTreeNode | None, int | None]:
         """Get the node containing the value and his parent node.
 
@@ -445,7 +446,7 @@ class BKTree[T]:
         for _, child in node.children.items():
             self.insert_subtree(child, parent_node)
 
-    def remove(self, value: T) -> None:
+    def remove(self, value: ValueT) -> None:
         if not self.root:
             return
 
@@ -514,19 +515,19 @@ class BKTree[T]:
             tree_copy.nb_elements = self.nb_elements
         return tree_copy
 
-    def collect_values(self) -> list[T]:
+    def collect_values(self) -> list[ValueT]:
         """Collect all the values in the BK-Tree.
 
         Returns:
             list[Song]: The list of the songs.
         """
-        values: list[T] = []
+        values: list[ValueT] = []
         if self.root:
             self._collect_values(self.root, values)
 
         return values
 
-    def _collect_values(self, node: BKTreeNode, values: list[T]):
+    def _collect_values(self, node: BKTreeNode, values: list[ValueT]):
         values.append(node.value)
 
         for child in node.children.values():
@@ -550,7 +551,7 @@ class BKTree[T]:
         for node in other_tree_node.children.values():
             self._combine(node)
 
-    def find_best_root_node(self) -> T | None:
+    def find_best_root_node(self) -> ValueT | None:
         """Find the root node with the best efficiency (which has the lowest average distance from other nodes)
 
         Returns:
